#!/usr/bin/perl -w
#translate ratio.txt of FREEC into BED track

use strict;

my $usage = qq{
    $0

    -----------------------------
    mandatory parameters:
    
    -cnv	filename        file with CNVs (FREEC output)
    -sv		filename        file with SVs (SVDetect output)
    
    -----------------------------
    optional parameters:
    
    -d		value (def. 0)	maximal distance between predicted breakpoints by SVDetect and FREEC 
    -v 				verbose mode
    -o		output directory (default ./ )
};

if(scalar(@ARGV) == 0){
    print $usage;
    exit(0);
}

## mandatory arguments

my $CNVfilename = "";
my $SVfilename = "";

my $verbose = 0;
my $maxDist = 0;
my $outDir = ".";

## parse command line arguments

while(scalar(@ARGV) > 0){
    my $this_arg = shift @ARGV;
    if ( $this_arg eq '-h') {print "$usage\n"; exit; }

    elsif ( $this_arg eq '-cnv') {$CNVfilename = shift @ARGV;}
    elsif ( $this_arg eq '-sv') {$SVfilename = shift @ARGV;}
    elsif ( $this_arg eq '-d') {$maxDist = shift @ARGV;}
    elsif ( $this_arg eq '-o') {$outDir = shift @ARGV;}

    elsif ( $this_arg eq '-v') {$verbose = 1;}

    elsif ( $this_arg =~ m/^-/ ) { print "unknown flag: $this_arg\n";}
}

if( $CNVfilename eq ""){
    die "you should specify file with CNVs\n";
}
if( $SVfilename eq ""){
    die "you should specify file with SVs \n";
}   
my $linksName = $SVfilename ;
$linksName =~ s/.*[\\\/]//g;

#-----------read file with CNVs-----
my $CNVfilenameOUT;



if ($CNVfilename =~ /(.*).gz$/) {
        open(FILE, "gunzip -c $CNVfilename |") or die "$0: can't open ".$CNVfilename.":$!\n";
	$CNVfilenameOUT=$1.".withLinksFrom.".$linksName;
} else {
        open (FILE, "<$CNVfilename") or die "Cannot open file $CNVfilename!!!!: $!";
	$CNVfilenameOUT=$CNVfilename.".withLinksFrom.".$linksName;
}

my ($chr,$start,$end,$CPN,$type);
my %breaks;
my %chromosomes;
my $count = 0;
my $countChrom = 0;

my $chrCount = 0;

while (<FILE>) {		
    chomp;
    #1	72765000	72812999	0	loss
    #1	85972000	86005999	4	gain
    
    ($chr,$start,$end,$CPN,$type) = split /\t/;
    
    unless (exists ($chromosomes{$chr})) {
	$chromosomes{$chr} = $chrCount;
	$chrCount++;
	$countChrom = 0;
    }
    
    #push(@{$breaks{$chr}},$start);
    #push(@{$breaks{$chr}},$end);
    
    $breaks{$chr}->[$countChrom]{name} = $_;
    $breaks{$chr}->[$countChrom]{1} = -1;
    $breaks{$chr}->[$countChrom]{2} = -1;
    $breaks{$chr}->[$countChrom]{start} = $start;
    $breaks{$chr}->[$countChrom]{end} = $end;

    $countChrom++;
    $count++;
}
close FILE;
$CNVfilename =~ s/.*[\\\/]//g;
print STDERR "Read: $CNVfilename,\t$count x 2 breakpoints\n";

#-----------read file with links-----
$count = 0;
  
if ($SVfilename =~ /.gz$/) {
        open(FILE, "gunzip -c $SVfilename |") or die "$0: can't open ".$SVfilename.":$!\n";
} else {
        open (FILE, "<$SVfilename") or die "Cannot open file $SVfilename!!!!: $!";
}

$SVfilename =~ s/.*[\\\/]//g;

my $output1 = $outDir."/1.".$maxDist.".".$SVfilename.".".$CNVfilename;
my $output2 = $outDir."/2.".$maxDist.".".$SVfilename.".".$CNVfilename;
my $output0 = $outDir."/0.".$maxDist.".".$SVfilename.".".$CNVfilename;

open (OUT1, ">$output1") or die "Cannot open file $output1!!!!: $!";
open (OUT2, ">$output2") or die "Cannot open file $output2!!!!: $!";
open (OUT0, ">$output0") or die "Cannot open file $output0!!!!: $!";

my (@chr, @flag,@breakpoint,$line);
my ($class1,$class2,$class0)=(0,0,0);

while (<FILE>) {
    $line = $_;
    chomp;
    #1	118708	119132	1	118868	119207	3	(>WUSI-EAS671_0003:2:94:7430:7218#0_3*...)	(F*,F,..)	(R*,R,..)	(1*,1,..)	(2*,2,..)	(1*,2,3..)	(2,1*,..)	(118325*,118708,...)	(118991*,118943,..)	S_DUP	188.6666	5/5	UNBAL	3/5	3/3	(119132,119079)	(118836,118868)	0.6	5	1.ni.noNearDup2.952	1.ni.noNearDup2.952
    ($flag[1],$flag[0])=(0,0);
    ($chr[0],$chr[1],$breakpoint[0],$breakpoint[1],$type) = (split /\t/)[0,3,20,21,16]; 
    unless ($breakpoint[0] =~m/\,/) {
	($breakpoint[0],$breakpoint[1])=(split /\t/,$line)[22,23];  #print "$breakpoint[0];$breakpoint[1] in\n$line\n";
    }
    $count++;
    my (@over,@selectedBreak);
    for my $i (0..1) {
	#(119132,119079) or (119132,119079),(119090,129090)
	$over[$i] = -1;
	my @local_breaks = split (/\),\(/,$breakpoint[$i]); 
	for my $break (@local_breaks) {
	    $break =~ s/[\)\(]//g;
	    my ($left,$right)=sort {$a <=> $b} split (/,/,$break);
	    my ($over_,$selectedBreak_) = overlapWithCNV($line,$left,$right,$breaks{$chr[$i]});
	    if ($over_>=0) {
		if ($flag[$i]==0) {
		    $flag[$i]=1 if ($over_>=0);	 
		    ($over[$i],$selectedBreak[$i]) = ($over_,$selectedBreak_) ; #print "found $over_\n";
		} else { print "found second! $over_\n";
		    if ($selectedBreak[$i] ne $selectedBreak_) {
			$flag[$i]=2; print "different breaks\n";
			$over[$i] = $over[$i].",".$over_;
		    }
		}			
	    }   
	}
    }
    chomp $line;

    if ($flag[1]+$flag[0]==0) {
	chomp $line;
	print OUT0 $line."\t".$over[0]."\t".$over[1]."\n";
	$class0++;
    } elsif ($flag[1]+$flag[0]==1) {
	print OUT1 $line."\t".$over[0]."\t".$over[1]."\n";
	$class1++;
    } elsif ($flag[1]+$flag[0]==2) {
	if ($selectedBreak[0] ne $selectedBreak[1] || $type eq "INV_DUPLI") {
	    print OUT2 $line."\t".$over[0]."\t".$over[1]."\n";
	    $class2++;
	} else {
	    print OUT1 $line."\t".$over[0]."\t".$over[1]."\n";
	    $class1++;    
	}
    } elsif ($flag[1]+$flag[0]==3) {
	print "should not happen!\n";
	print "$line\n";
	if ($selectedBreak[0] ne $selectedBreak[1] || $type eq "INV_DUPLI") {
	    print OUT2 $line."\t".$over[0]."\t".$over[1]."\n";
	    $class2++;
	} else {
	    print OUT1 $line."\t".$over[0]."\t".$over[1]."\n";
	    $class1++;    
	}
    }
    
}

close FILE;
close OUT1;
close OUT2;
close OUT0;

print STDERR "Read: $SVfilename,\t$count links: 0 $class0, 1 $class1, 2 $class2\n";


sub overlapWithCNV {
    my ($line,$left,$right,$cnvBreak) = @_; #print "($left,$right,$cnvBreak)\n";
    my $fpos = $left-$maxDist;
    my $lpos = $right+$maxDist;
    my $valueToReturn = -1;
    my $selectedBreak = -1;
    for my $break (@{$cnvBreak}) {
	my $value = -1;
	if ($break->{start}>=$fpos && $break->{start}<=$lpos) {
	    #print "found!\t";
	    if ($break->{start}>=$left && $break->{start}<=$right) {
		$value = 0 ; #print "exact!\t";
	    } else {
		$value = min(abs($break->{start}-$left),abs($break->{start}-$right));
	    } 
	    if ($break->{1}==-1 || $break->{1}>$value) {
		$break->{1}=$value;
		$break->{link1} = $line;
		
	    }
	    if ($valueToReturn ==-1 || $valueToReturn>$value) {
		$valueToReturn=$value; 
		$selectedBreak=$break;
	    }
	}
	$value = -1;
	if ($break->{end}>=$fpos && $break->{end}<=$lpos) {
	    if ($break->{end}>=$left && $break->{end}<=$right) {
		$value = 0 ;
	    } else {
		$value = min(abs($break->{end}-$left),abs($break->{end}-$right));
	    }
	    if ($break->{2}==-1 || $break->{2}>$value) {
		$break->{2}=$value;
		$break->{link2} = $line;		
	    }
	    if ($valueToReturn ==-1 || $valueToReturn>$value) {
		#print "found better ($valueToReturn) value for the end: $value\n";
		$valueToReturn=$value;
		$selectedBreak=$break; 
	    }
	}	
    }
   return ($valueToReturn,$selectedBreak);    
}

open (OUT, ">$CNVfilenameOUT") or die "Cannot open file $CNVfilenameOUT!!!!: $!";
    for my $chrom (sort keys %chromosomes) {
	for my $i (0..$chromosomes{$chrom}) {
	    if ($breaks{$chrom}->[$i]{name}=~/[0-9]/) {
		if ($breaks{$chrom}->[$i]{1}>=0) {
		    chomp $breaks{$chrom}->[$i]{link1};
		    print OUT  $breaks{$chrom}->[$i]{name}."\t".$breaks{$chrom}->[$i]{1}."\t".$breaks{$chrom}->[$i]{link1}."\n";
		    
		} else {
		    print OUT  $breaks{$chrom}->[$i]{name}."\t".$breaks{$chrom}->[$i]{1}."\n";	
		}	
		if ($breaks{$chrom}->[$i]{2}>=0) {
		    chomp $breaks{$chrom}->[$i]{link2};
		    print OUT  $breaks{$chrom}->[$i]{name}."\t".$breaks{$chrom}->[$i]{2}."\t".$breaks{$chrom}->[$i]{link2}."\n";		
		} else {
		    print OUT  $breaks{$chrom}->[$i]{name}."\t".$breaks{$chrom}->[$i]{2}."\n";	
		}		    
	    }	    
	}	
    }
close  OUT;

sub overlapWithCNV_ { #depr
    my ($fpos,$lpos,$cnvBreak) = @_;    
    for my $break (@{$cnvBreak}) {
	if ($break>=$fpos && $break<=$lpos) {
		return 1;
	}		
    }
   return 0;    
}

sub min {
    my ($a,$b)= @_;
    return $a if $a <= $b;
    return $b;    
}
