#!/usr/bin/perl -w

use strict;

my $usage = qq{
    $0 - merges overlapping links

    -----------------------------
    mandatory parameters:
    
    -l   list                   comma-delimeted list of files with links
 
    -----------------------------
    optional parameters:
    
    -v 				verbose mode
    -t 				check link type

 
};

if(scalar(@ARGV) == 0){
    print $usage;
    exit(0);
}

## mandatory arguments

my $filename = "";

my $INFINITY = 10000000000;
## optional arguments
my $verbose = 0;
my $listOfFiles = "";
my $shouldBeSameType = 0;
## parse command line arguments

while(scalar(@ARGV) > 0){
    my $this_arg = shift @ARGV;
    if ( $this_arg eq '-h') {print "$usage\n"; exit; }

    elsif ( $this_arg eq '-v') {$verbose = 1;}
    elsif ( $this_arg eq '-t') {$shouldBeSameType = 1;}
    elsif ( $this_arg eq '-l') {$listOfFiles = shift @ARGV;}      

    elsif ( $this_arg =~ m/^-/ ) { print "unknown flag: $this_arg\n";}
}

if( $listOfFiles eq ""){
    die "you should specify file with pairs of a list of files\n";
}


my %listOfFiles;

if ($listOfFiles ne "") {
    my @list = split(",",$listOfFiles);
    for my $file (@list) {
        my $name = $file;
        $name =~ s/.*\///; 
        if ($name =~ m/_([A-Z0-9\-]+)[_-]/) {
            $name = $1;
            $listOfFiles{$name}=$file;   
        } elsif ($name =~ m/(.*)\.int/) {
            $name = $1;
            $listOfFiles{$name}=$file;        
        }
    }
}

#-----------read files with pairs-----

my %links;
for my $name (sort keys %listOfFiles) {
    my $filename = $listOfFiles{$name};  
    
    my $numberOfInitialSites = 0;
    my $sitesLeft = 0;
    
    
    if ($filename eq "") {
            print "No file with pairs!\n" if ($verbose) ;
    }
    else {
            open (FILE, "<$filename") or die "Cannot open file $filename!!!!: $!";
            print "reading $filename\nname: $name\n" if ($verbose);

            while (<FILE>) {		
    
                chomp;			
                my $entry = $_;
                my @a = split /\t/;			

                my $sitesChr1 = "chr".$a[0];
                my $firstPos1 = $a[1];
                my $lastPos1 = $a[2];
                my $sitesChr2 = "chr".$a[3];
                my $firstPos2 = $a[4];
                my $lastPos2 = $a[5];
                my $pos1 = ($firstPos1 + $lastPos1)/2;
                my $pos2 = ($firstPos2 + $lastPos2)/2;
                my $strand1 = $a[8];
                my $type = $a[16];
                my $isrec = 0;
                #exit if ($numberOfInitialSites > 84);
                #print "line $numberOfInitialSites :",$firstPos1," ",$lastPos1, " ",$firstPos2, " ", $lastPos2, "\n";
                
                if (exists ($links{$name}->{$sitesChr1}{$sitesChr2})) {
                    for my $key (keys %{$links{$name}->{$sitesChr1}{$sitesChr2}}) {
                        if (($links{$name}->{$sitesChr1}{$sitesChr2}{$key}{start1} == $firstPos1 ||  $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{end1} == $lastPos1)&& overlap ($links{$name}->{$sitesChr1}{$sitesChr2}{$key}{start2},$links{$name}->{$sitesChr1}{$sitesChr2}{$key}{end2},$firstPos2,$lastPos2) && $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{type} eq $type) {
                            my ($newEntry,$start1,$end1,$start2,$end2)  = recalculate ($entry,$links{$name}->{$sitesChr1}{$sitesChr2}{$key}{entry});
                            #print "newEntry:\n $newEntry\n";
                            $isrec = 1;
                            $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{type} = $type;
                            $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{entry} = $newEntry;
                            $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{start1} = $start1;
                            $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{end1} = $end1;
                            $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{start2} = $start2;
                            $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{end2} = $end2;                            
                        } elsif (($links{$name}->{$sitesChr1}{$sitesChr2}{$key}{start2} == $firstPos2 ||  $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{end2} == $lastPos2)&& overlap ($links{$name}->{$sitesChr1}{$sitesChr2}{$key}{start1},$links{$name}->{$sitesChr1}{$sitesChr2}{$key}{end1},$firstPos1,$lastPos1) && $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{type} eq $type) {
                            my ($newEntry,$start1,$end1,$start2,$end2)  = recalculate ($entry,$links{$name}->{$sitesChr1}{$sitesChr2}{$key}{entry});
                            $isrec = 1;
                           $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{type} = $type;
                            $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{entry} = $newEntry;
                            $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{start1} = $start1;
                            $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{end1} = $end1;
                            $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{start2} = $start2;
                            $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{end2} = $end2;   
                        } elsif(($links{$name}->{$sitesChr1}{$sitesChr2}{$key}{start1} == $firstPos1 ||  $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{end1} == $lastPos1)&& overlap ($links{$name}->{$sitesChr1}{$sitesChr2}{$key}{start2},$links{$name}->{$sitesChr1}{$sitesChr2}{$key}{end2},$firstPos2,$lastPos2) && $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{type} ne $type) {
                            #print "HERE! $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{type} ne $type \n";
                        }
                        
                    }
                }
                 
                unless ($isrec) {
                    my $key = $firstPos1.$type.$lastPos2;
                    $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{type} = $type;
                    $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{entry} = $entry;
                    $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{start1} = $firstPos1;
                    $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{end1} = $lastPos1;
                    $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{start2} = $firstPos2;
                    $links{$name}->{$sitesChr1}{$sitesChr2}{$key}{end2} = $lastPos2;
                    $sitesLeft++;
                }                
                    
                $numberOfInitialSites++;
            }
            close FILE;
            print "\t$filename is read!\n$numberOfInitialSites pairs, $sitesLeft is kept\n" if ($verbose) ;
    
    }
    
}

for my $name (sort keys %listOfFiles) {
    my $resultFileName = $listOfFiles{$name}.".merged";
    open (OUT , ">$resultFileName") or die "Cannot open file $resultFileName!!!!: $!";

    my $filename = $listOfFiles{$name};  
    my $controlCount = 0;
    my $common = 0;
    for my $chr1 (keys %{$links{$name}}) {
        for my $chr2 (keys %{$links{$name}->{$chr1}}) {
            for my $key (keys %{$links{$name}->{$chr1}{$chr2}}) {
                #print "here: $links{$name}->{$chr1}{$chr2}{$key}{start1},$links{$name}->{$chr1}{$chr2}{$key}{end1}\n";

                my $tag = $name;
                for my $otherSample (keys %listOfFiles) {
                    unless ($otherSample eq $name) {
                        my $ifOverlap = 0;
                        #check if the link overlap any other link:
                        for my $newkey (keys %{$links{$otherSample}->{$chr1}{$chr2}}) {
                            #print "here2: $links{$otherSample}->{$chr1}{$chr2}{$newkey}{start1},$links{$otherSample}->{$chr1}{$chr2}{$newkey}{end1}\n";

                            if (($shouldBeSameType==0 || ($shouldBeSameType==1 && $links{$name}->{$chr1}{$chr2}{$key}{type} eq $links{$otherSample}->{$chr1}{$chr2}{$newkey}{type}) ) && overlap($links{$name}->{$chr1}{$chr2}{$key}{start1},$links{$name}->{$chr1}{$chr2}{$key}{end1},$links{$otherSample}->{$chr1}{$chr2}{$newkey}{start1},$links{$otherSample}->{$chr1}{$chr2}{$newkey}{end1}) &&
                                overlap($links{$name}->{$chr1}{$chr2}{$key}{start2},$links{$name}->{$chr1}{$chr2}{$key}{end2},$links{$otherSample}->{$chr1}{$chr2}{$newkey}{start2},$links{$otherSample}->{$chr1}{$chr2}{$newkey}{end2})) {
                                $ifOverlap = 1;
                                last;
                            }
                        }
                        if ($ifOverlap) {
                            $tag.="/$otherSample";                            
                            #print $tag,"\n" if ($verbose);
                        }
                    }
                }
                $common++ if ($tag =~ /\//);
                print OUT $links{$name}->{$chr1}{$chr2}{$key}{entry}, "\t$tag\n";                
                $controlCount++;
            }
        }
        
    }
    print "printed $controlCount lines\namong them common: $common\n" if ($verbose) ;

    close OUT ;
}


sub almostEqual {
	my ($a,$b) = ($_[0],$_[1]);
	if (abs($a-$b)<=1000) {
		return 1;
	}
	return 0;
}


#functions which were not used here
sub find {
	my ($name,@array) = ($_[0],@{$_[1]});
	for my $i (0..@array-1) {
		return $i if ($name eq $array[$i]);
	}
	return -1;
}



sub search_all {
		
	my ($gen_chro,$gen_fpos,$gen_lastPos,$gen_strand,$sitesChr,$firstPos,$lastPos) = @_;
	#my $dist = 1000000000000;
	
	
	for my $i (0..scalar(@$sitesChr)-1) {
		
		if ($gen_chro eq $sitesChr->[$i]) {
					
			my $sitePos = ($lastPos->[$i]-$firstPos->[$i])/2+$firstPos->[$i];
			#my $dist_tmp = min(abs($gen_lastPos-$sitePos),abs($gen_fpos-$sitePos));
			my $dist;
			if ($gen_strand eq '+') {
				$dist = $sitePos - $gen_fpos;
			}
			elsif ($gen_strand eq '-') {
				$dist =  $gen_lastPos - $sitePos ;
			}		
			
			
			print "\t$dist";
		}
		else {
			print "\tN/A";
		}
	}		
}

sub search {
	my  $dir;
	
	
	my ($gen_chro,$gen_fpos,$gen_lastPos,$gen_strand,$sitesChr,$firstPos,$lastPos) = @_;
	my $dist = 1000000000000;
	
	
	#if ($gen_chro eq 'chr14') {
	#		print "chr14! $gen_fpos\n";
	#}
	for my $i (0..scalar(@$sitesChr)-1) {
		#if (($gen_chro eq 'chr14')&&($gen_fpos==103240485)) {
			#print "Here!!!! $sitesChr->[$i]!\n";
		#}
		if ($gen_chro eq $sitesChr->[$i]) {
		
			#if (($gen_chro eq 'chr14')&&($gen_fpos==103240485)) {
				#print "Here!!!! $gen_chro!\n";
			#}
		
			my $sitePos = ($lastPos->[$i]-$firstPos->[$i])/2+$firstPos->[$i];
			#my $dist_tmp = min(abs($gen_lastPos-$sitePos),abs($gen_fpos-$sitePos));
			my $dist_tmp = - $gen_fpos+$sitePos;
			if (abs($dist_tmp)<abs($dist)) {
				
				if ($dist_tmp <= 0) {
					if ($gen_strand eq '+') {
						#print "+";
						$dir = "upstream";
						$dist = $dist_tmp;
					}
					else {
						$dir = "downstream";
						$dist = -$dist_tmp;
						#print "-";
					}
				}
				else{
					if ($gen_strand eq '+') {
						#print "+";
						$dir = "downstream";
						$dist = $dist_tmp;
					}
					else {
						#print "-";
						$dir = "upstream";
						$dist = -$dist_tmp;
					}				
				}
				#if (($gen_lastPos>$sitePos)&&($gen_fpos<$sitePos)) {
				#	$dist = 0;
				#	$dir = "middle";
				#}
			
			}
		}
	}
	#if (($gen_chro eq 'chr14')&&($gen_fpos==103240485)) {
	#		print "Here $dist, $dir!\n";
	#}
	
	return ($dist, $dir);
}
sub min {
	return $_[0] if $_[1] eq "";
        return $_[1] if $_[0] eq "";
        
        return $_[0] if ($_[0]<$_[1]);
	$_[1];
}

sub max {
    	return $_[0] if $_[1] eq "";
        return $_[1] if $_[0] eq "";
	return $_[0] if ($_[0]>$_[1]);
	$_[1];
}

sub largeMin {
    my @a = @_;
    my $min = $a[0];
    for my $i (1..scalar(@a)-1) {
        $min = min ($a[$i], $min);
    }
    return $min;
}

sub largeMax {
    my @a = @_;
    my $max = $a[0];
    for my $i (1..scalar(@a)-1) {
        $max = max ($a[$i], $max);
    }
    return $max;
}

sub overlap {
    my($cs_hit,$ce_hit,$cs_region,$ce_region)=@_;
    if( ($ce_hit < $cs_region ) || ($cs_hit > $ce_region)  ) {
        return 0;
    }
    return 1;
}

sub recalculate  {
    my ($entry1,$entry2) = @_;
    my ($newEntry,$start1,$end1,$start2,$end2) ;
    
    my @pairsToAdd;
    
    my @a = split /\t/, $entry1;		

    my $sitesChr1 = $a[0];
    my $sitesChr2 = $a[3];

    my $p1 = $a[7];
    $p1 =~ s/[\)\(]//g;
    $p1 =~ s/\d+\*//g;   

    my @pairs1 = split (",", $p1);
    
    #print keys %hash1,"\n";
    my @b = split /\t/, $entry2;
    my $p2 = $b[7];
    $p2 =~s/[\)\(]//g;    $p2 =~ s/\d+\*//g;   

    my @pairs2 = split (",", $p2);
    #print "@pairs2\n";
    my $count = 0;
    my $addToLeft = 1;
    
    if (scalar(@pairs2) <= scalar(@pairs1)) {
        my %hash1;
        @hash1{@pairs1}=@pairs1;
        for my $pairName (@pairs2) {
            unless (exists ($hash1{$pairName})) {
                push (@pairsToAdd,$count);
            }
            $count++;
        }        
    } else {
        $addToLeft = 0;
        my %hash1;
        @hash1{@pairs2}=@pairs2;
        for my $pairName (@pairs1) {
           unless (exists ($hash1{$pairName})) {
               push (@pairsToAdd,$count);
           }
           $count++;
       }       
    }
     
    #print "to add:\n",scalar(@pairsToAdd),"\n",scalar(@pairs1)," vs ",scalar(@pairs2),"\n";
    $p1 = $a[14];
    $p1 =~ s/[\)\(]//g;    $p1 =~ s/\d+[\*\$\@]//g;   

                # print "p1:\n$p1\n";
    my @positionsStart1 = split (",", $p1);
    $p1 = $a[15];
    $p1 =~ s/[\)\(]//g;    $p1 =~ s/\d+[\*\$\@]//g;   

    my @positionsEnd1 = split (",", $p1);
    
    $p1 = $b[14];
    $p1 =~ s/[\)\(]//g;    $p1 =~ s/\d+[\*\$\@]//g;   

    my @positionsStart2 = split (",", $p1);
    $p1 = $b[15];
    $p1 =~ s/[\)\(]//g;    $p1 =~ s/\d+[\*\$\@]//g;   

    my @positionsEnd2 = split (",", $p1);
    
    if ( $addToLeft) {
       
        for my $colomn (7..15) {
            $b[$colomn] =~ s/[\)\(]//g;     

            my @array = (split(",", $b[$colomn]))[@pairsToAdd];
            $a[$colomn] = $a[$colomn].",".join(",",@array);
        }
       
        $a[6] = "=".$a[6]."+".scalar(@pairsToAdd);
        $a[6] =~ s/[\=]+/=/;

    } else {
        
        for my $colomn (7..15) {
            $a[$colomn] =~ s/[\)\(]//g;
            my @array = (split(",", $a[$colomn]))[@pairsToAdd];
            $b[$colomn] = $b[$colomn].",".join(",",@array);
            
        }
        $b[6] = "=".$b[6]."+".scalar(@pairsToAdd);
        $b[6] =~ s/[\=]+/=/;
    }
   
    
    $start1 = largeMin(@positionsStart1,@positionsStart2);
    
    #print "positionsStart1:\n";
    #print "@positionsStart1\n";
    #print "positionsStart2:\n";
    #print "@positionsStart2\n";
    #print "start1 $start1\n";
    $end1 =  largeMax(@positionsStart1,@positionsStart2);
    $start2 = largeMin(@positionsEnd1,@positionsEnd2);
    $end2= largeMax(@positionsEnd1,@positionsEnd2);
    
     if ( $addToLeft) {
        $a[1] = $start1;
        $a[2] = $end1;
        $a[4] = $start2;
        $a[5] = $end2;
            
        $newEntry = join("\t",@a);
    }else {
        $b[1] = $start1;
        $b[2] = $end1;
        $b[4] = $start2;
        $b[5] = $end2;
            
        $newEntry = join("\t",@b);
    }
    
    return ($newEntry,$start1,$end1,$start2,$end2) ;
    
}
